package KBFile;

import java.io.*;
import java.util.*;
/*
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 */

public class ATObjectFile extends BaseRecordsFile {

  /**
   * Hashtable which holds the in-memory index. For efficiency, the entire index 
   * is cached in memory. The hashtable maps a key of type String to a RecordHeader.
   */
  protected Map memIndex = null;    

  /**
   * Creates a new database file.  The initialSize parameter determines the 
   * amount of space which is allocated for the index.  The index can grow 
   * dynamically, but the parameter is provide to increase 
   * efficiency. 
   */
  public ATObjectFile(String dbPath, int initialSize) throws IOException, RecordsFileException {
	super(dbPath, initialSize);
	memIndex = new HashMap(initialSize);
  }  
  /**
   * Opens an existing database and initializes the in-memory index. 
   */
  public ATObjectFile(String dbPath, String accessFlags) throws IOException, RecordsFileException {
	super(dbPath, accessFlags);
	int numRecords = readNumRecordsHeader();
	System.out.println("Number of records in index is " + numRecords);
	memIndex = new HashMap(numRecords);
	for (int i = 0; i < numRecords; i++) {
	  String key = readKeyFromIndex(i);
	  RecordHeader header = readRecordHeaderFromIndex(i);
	  header.setIndexPosition(i);
	  memIndex.put(key, header);
	}
  }  
  /**
   * Adds the new record to the in-memory index and calls the super class add
   * the index entry to the file. 
   */
  protected void addEntryToIndex(String key, RecordHeader newRecord, int currentNumRecords) throws IOException, RecordsFileException {
	super.addEntryToIndex(key, newRecord, currentNumRecords);
	memIndex.put(key, newRecord);   
  }  
  /**
   * This method searches the file for free space and then returns a RecordHeader 
   * which uses the space. (O(n) memory accesses)
   */
  protected RecordHeader allocateRecord(String key, int dataLength) throws RecordsFileException, IOException {
	// search for empty space
	RecordHeader newRecord = null;
	for (Iterator i= memIndex.values().iterator(); i.hasNext();) {
	  RecordHeader next = (RecordHeader)i.next();
	  int free = next.getFreeSpace();
	  if (dataLength <= next.getFreeSpace()) {
		newRecord = next.split();
		writeRecordHeaderToIndex(next);
		break;
	  }
	}
	if (newRecord == null) {
	  // append record to end of file - grows file to allocate space
	  long fp = getFileLength();
	  setFileLength(fp + dataLength);
	  newRecord = new RecordHeader(fp, dataLength);
	} 
	return newRecord;
  }  
  /**
   * Closes the database. 
   */
  public synchronized void close() throws IOException, RecordsFileException {
	try {
	  super.close();
	} finally {
	  memIndex.clear();
	  memIndex = null;
	}
  }  
  /**
   * Removes the record from the index. Replaces the target with the entry at the 
   * end of the index. 
   */
  protected void deleteEntryFromIndex(String key, RecordHeader header, int currentNumRecords) throws IOException, RecordsFileException {
	super.deleteEntryFromIndex(key, header, currentNumRecords);
	RecordHeader deleted = (RecordHeader)memIndex.remove(key);
  }  
  /**
   * Returns the current number of records in the database. 
   */
  public synchronized int getNumRecords() {
	return memIndex.size();
  }  
  /**
   * Returns the record to which the target file pointer belongs - meaning the specified location
   * in the file is part of the record data of the RecordHeader which is returned.  Returns null if 
   * the location is not part of a record. (O(n) mem accesses)
   */
  protected RecordHeader getRecordAt(long targetFp) throws RecordsFileException {
	for (Iterator i= memIndex.values().iterator(); i.hasNext();) {
	  RecordHeader next = (RecordHeader)i.next(); 
	  if (targetFp >= next.dataPointer && targetFp < next.dataPointer + (long)next.dataCapacity)
			return next;
	}
	return null;
  }  
  /**
   * Maps a key to a record header by looking it up in the in-memory index.
   */
  protected RecordHeader keyToRecordHeader(String key) throws RecordsFileException {
	RecordHeader h = (RecordHeader)memIndex.get(key);
	if (h==null) {
	  throw new RecordsFileException("Key not found: " + key);
	} 
	return h;
  }  
  /**
   * Returns an enumeration of all the keys in the database.
   */
  public synchronized Collection listKeys() {
	//return (Set)memIndex.keySet();
	return (Collection)memIndex.keySet();
  }  
  /**
   * Checks if there is a record belonging to the given key. 
   */
  public synchronized boolean recordExists(String key) {
	System.out.println(" ** Looking in hashtable " + key);
	return memIndex.containsKey(key);
  }  
}
