package Transforms;

/**
 * This class represents the Agent Component Mobility Transformation
 * (ACMT4) in the Mobile Design transformation process.
 * This transform adds five transitions between the reestablish, start,
 * end, idle and tryMove states to the agent component.  
 *
 * Creation date: (12/28/00 12:07:00 AM)
 * @author: Athie Self
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */
 
import java.util.Vector;
import system.*;

public class ACMTransform4 extends Transform {
	TSystem	currentSys = null;	// A pointer to the system being transformed
	TSystem beforeSys = null; 	// A copy of the system before the transformation
	TSystem sys = null;			// A copy of the system that is used to work on
/**
 * ACMTransform4 constructor comment.
 */

public ACMTransform4(TSystem t) {
	super();

	sys = t.copy();
	currentSys = t;
	beforeSys = t.copy();
	setRationale("");
	if(applicable(sys, null)){
		execute(null);
	}
}
/**
 * This method will only return true if the target is a TSystem 
 * and that system has roles, tasks, and protocols defined.
 *
 * Creation date: (11/6/00 8:52:43 AM)
 */
public static boolean applicable(Object target, Object params) {
	TSystem t;
	if(target instanceof TSystem){
		t = (TSystem)target;
		if((t.roles != null)
			&& (t.tasks != null)){
				return true;
			}
		else {
			return false;
		}
	}
	else {
		return false;
	}
}
/**
 * the params attribute is not used
 */

public void execute(Object params) {	

	Vector  agents = sys.agents;
	
	for(int i = 0; i < agents.size(); i++){
		
		Agent ag = (Agent)agents.elementAt(i);

		if (isMobility_Specified(ag)) {
			
			// get the AgentComponent
			ATagent atAgent = ag.getAtAgent();
			ATarchitecture atArch = atAgent.getArchitecture();
			ATagentComponent atagentComponent = atArch.findComponentbyname("AgentComponent");

			// get the state table for the AgentComponent
			ATstatetable atStateTable = atagentComponent.getStateTable();
			ATtableElement atStartState = atStateTable.findStatebyname("StartState");
			ATtableElement atEndState = atStateTable.findStatebyname("EndState");
			ATtableElement atReestablish = atStateTable.findStatebyname("reestablish");
			ATtableElement atIdle = atStateTable.findStatebyname("idle");
			ATtableElement atTryMove = atStateTable.findStatebyname("tryMove");
			
			// add transitions to the state table
			ATtransition atTransition1 = new ATtransition(sys.version,sys.atSys.getNextKey());
			atTransition1.setGuardCondition("moved");
			atTransition1.setCoordinates(atTryMove,atEndState);
			atStateTable.addTransition(atTransition1);
			
			ATtransition atTransition2 = new ATtransition(sys.version,sys.atSys.getNextKey());
			atTransition2.setGuardCondition("NOT moved");
			atTransition2.setCoordinates(atTryMove,atReestablish);
			atStateTable.addTransition(atTransition2);

			ATtransition atTransition3 = new ATtransition(sys.version,sys.atSys.getNextKey());
			atTransition3.setGuardCondition("NOT compsStarted");
			atTransition3.setCoordinates(atReestablish,atEndState);
			atStateTable.addTransition(atTransition3);

			ATtransition atTransition4 = new ATtransition(sys.version,sys.atSys.getNextKey());
			atTransition4.setGuardCondition("compsStarted");
			atTransition4.setCoordinates(atReestablish,atIdle);
			atStateTable.addTransition(atTransition4);

			ATtransition atTransition5 = new ATtransition(sys.version,sys.atSys.getNextKey());
			atTransition5.setGuardCondition("stateInfo!=null");
			atTransition5.setCoordinates(atStartState,atReestablish);
			atStateTable.addTransition(atTransition5);

			atagentComponent.setStateTable(atStateTable);

		}	
	}

	// now make the altered system the current system
	// can be undone by the unDo method
	currentSys = sys;
	
}
/**
 * Insert the method's description here.
 * Creation date: (12/29/2000 9:53:42 AM)
 * @return boolean
 * @param a Transforms.Agent
 */

public boolean isMobility_Specified(Agent a) {

	Vector actions    = new Vector();
	Vector states     = new Vector();
	
	Vector components = a.getComponents();
	
	if(components != null) {

		for(int j = 0; j < components.size(); j++) {

			Component c = (Component)components.elementAt(j);

			if (!c.getName().equals("AgentComponent")) {

				StateTable st = c.getStateTable();

				if(st != null) {
					states = st.getStates();
				}

				if(states != null) {					

					for(int k = 0; k < states.size(); k++) {

						State s = (State)states.elementAt(k);

						actions = s.getActions();

						if(actions != null) {

							for(int l = 0; l < actions.size(); l++) {

								Action act = (Action)actions.elementAt(l);

								FunctionCall fc = (FunctionCall)act.getFunction();

								if(fc!=null) {

									if(fc.getName().equals("move")) {
										return true;
									}
								}	
							}		
						}	
					}
				}
			}
		}
	}
	
	return false;	
}
/**
 * replay method comment.
 */
public void replay(Object top) {}
/**
 * unDo method comment.
 */
public void unDo(Object params) {
	currentSys = beforeSys;
}
}
