package Transforms;

/**
 * This class represents the Component Mobility Transformation
 * (CMT7) in the Mobile Design transformation process.
 * This transform adds a state and transition to partially handle
 * the process of components, belonging to a mobile agent, receiving
 * move required messages from the agent component.
 *
 * Creation date: (12/28/00 12:07:00 AM)
 * @author: Athie Self
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */

import java.util.Vector;
import system.*;

public class CMTransform7 extends Transform {
	TSystem	currentSys = null;	// A pointer to the system being transformed
	TSystem beforeSys = null; 	// A copy of the system before the transformation
	TSystem sys = null;			// A copy of the system that is used to work on
/**
 * CMTransform7 constructor comment.
 */

public CMTransform7(TSystem t) {
	super();

	sys = t.copy();
	currentSys = t;
	beforeSys = t.copy();
	setRationale("");
	if(applicable(sys, null)){
		execute(null);
	}
}
/**
 * This method will only return true if the target is a TSystem 
 * and that system has roles and tasks.
 *
 * Creation date: (11/6/00 8:52:43 AM)
 */
public static boolean applicable(Object target, Object params) {
	TSystem t;
	if(target instanceof TSystem){
		t = (TSystem)target;
		if((t.roles != null)
			&& (t.tasks != null)){
				return true;
			}
		else {
			return false;
		}
	}
	else {
		return false;
	}
}
/**
 * the params attribute is not used
 */

public void execute(Object params) {	
			
	Vector agents = sys.agents;
	
	boolean hasTransientMobileComponent = false;
	boolean isMobileComponent = false;
	
	int intMobileComponentCount = 0;
		
	for(int i = 0; i < agents.size(); i++) {
	
		Agent ag = (Agent)agents.elementAt(i);
		
		if (isMobility_Specified(ag)) {
			
			intMobileComponentCount = mobileComponentCount(ag);
						
			ATagent atAgent = ag.getAtAgent();
			ATarchitecture atArch = atAgent.getArchitecture();

			Vector components = ag.getComponents();

			for(int j = 0; j < components.size(); j++) {

				Component c = (Component)components.elementAt(j);

				if (!(c.getName().equals("AgentComponent"))) {

					if (isTransientComponent(c)) {
					
						hasTransientMobileComponent = isMobility_Specified_Component(c);
						
					}
				}
			}
			
			for(int k = 0; k < components.size(); k++) {

				Component c = (Component)components.elementAt(k);

				if (!(c.getName().equals("AgentComponent"))) {

					isMobileComponent = isMobility_Specified_Component(c);

					if (((intMobileComponentCount > 1) || (hasTransientMobileComponent))
						|| ((intMobileComponentCount == 1) && (!isMobileComponent))) {

						ATagentComponent atComponent = (ATagentComponent)c.getAtComponent();
						ATstatetable atStateTable = atComponent.getStateTable();
						ATtableElement atEndState = atStateTable.findStatebyname("EndState");
						ATtableElement atRestoreState = atStateTable.findStatebyname("restore");
						
						// add state to the state table
						ATstate atMoveReceivedState = new ATstate(sys.version,sys.atSys.getNextKey());
						atMoveReceivedState.setName("moveReceived");
						atMoveReceivedState.setAction("");
						atMoveReceivedState.setAction("stateInfo=saveCompState()");
						atMoveReceivedState.setAction("comp=getCompName()");
						atMoveReceivedState.setCoordinates(600,550,10,10);
						atStateTable.addState(atMoveReceivedState);
						
						ATtransition atTransition = new ATtransition(sys.version,sys.atSys.getNextKey());
						atTransition.setRecMessage("ready(stateInfo,comp)");
						atTransition.setCoordinates(atMoveReceivedState,atEndState);
						atStateTable.addTransition(atTransition);

						atComponent.setStateTable(atStateTable);
					}
				}
			}
		}
	}
							
	// now make the altered system the current system
	// can be undone by the unDo method
	currentSys = sys;
	
}
/**
 * Insert the method's description here.
 * Creation date: (12/29/2000 9:53:42 AM)
 * @return boolean
 * @param a Transforms.Agent
 */

public boolean isMobility_Specified(Agent a) {

	Vector actions    = new Vector();
	Vector states     = new Vector();
	
	Vector components = a.getComponents();
	
	if(components != null) {

		for(int j = 0; j < components.size(); j++) {

			Component c = (Component)components.elementAt(j);

			if (!c.getName().equals("AgentComponent")) {

				StateTable st = c.getStateTable();

				if(st != null) {
					states = st.getStates();
				}

				if(states != null) {					

					for(int k = 0; k < states.size(); k++) {

						State s = (State)states.elementAt(k);

						actions = s.getActions();

						if(actions != null) {

							for(int l = 0; l < actions.size(); l++) {

								Action act = (Action)actions.elementAt(l);

								FunctionCall fc = (FunctionCall)act.getFunction();

								if(fc!=null) {

									if(fc.getName().equals("move")) {
										return true;
									}
								}	
							}		
						}	
					}
				}
			}
		}
	}
	
	return false;	
}
/**
 * Insert the method's description here.
 * Creation date: (12/29/2000 9:53:42 AM)
 * @return boolean
 * @param c Transforms.Component
 */

public boolean isMobility_Specified_Component(Component c) {

	Vector			actions = new Vector();
	Vector			states = new Vector();
	StateTable		st;
	Action			act;
	State			s = null;
	FunctionCall	fc;

	st = c.getStateTable();
	if(st != null) {
		states = st.getStates();
	}
	if(states != null) {					
		for(int k = 0; k < states.size(); k++) {
			s = (State)states.elementAt(k);
			actions = s.getActions();
			if(actions != null) {
				for(int l = 0; l < actions.size(); l++) {
					act = (Action)actions.elementAt(l);
					fc = act.getFunction();
					if(fc!=null) {
						if(fc.getName().equals("move")) {
							return true;
						}
					}	
				}		
			}	
		}
	}
	
	return false;	
}
/**
 * Insert the method's description here.
 * Creation date: (12/29/2000 9:53:42 AM)
 * @return boolean
 * @param c Transforms.Component
 */

public boolean isTransientComponent(Component c) {

	Vector transitions = new Vector();

	if (!c.getName().equals("AgentComponent")) {
				
		StateTable st = (StateTable)c.getStateTable();
				
		if(st != null) {
			transitions = st.getTransitions();
		}
				
		if(transitions != null) {
			
			for(int k = 0; k < transitions.size(); k++) {
				
				Transition t = (Transition)transitions.elementAt(k);
				
				if ((t.getFromName().equals("StartState"))
					&& ((t.getReceiveEvent() != null) 
					|| (t.getReceive() != null))) {

					return true;
						
				}
			}
		}
	}
	
	return false;	
}
/**
 * Insert the method's description here.
 * Creation date: (12/29/2000 9:53:42 AM)
 * @return count of mobile components within an agent class
 * @param a Transforms.Agent
 */

public int mobileComponentCount(Agent a) {	
			
	int intMCCount = 0;
		
	ATagent atAgent = a.getAtAgent();
	ATarchitecture atArch = atAgent.getArchitecture();

	Vector components = a.getComponents();

	for(int j = 0; j < components.size(); j++) {

		Component c = (Component)components.elementAt(j);

		if (!(c.getName().equals("AgentComponent"))) {

			if (isMobility_Specified_Component(c)) {

				intMCCount++;
				
			}
		}
	}
							
	return intMCCount;
	
}
/**
 * replay method comment.
 */
public void replay(Object top) {}
/**
 * unDo method comment.
 */
public void unDo(Object params) {
	currentSys = beforeSys;
}
}
