package Transforms;

/**
 * This class represents the Component Mobility Transformation
 * (CMT9) in the Mobile Design transformation process.
 * This transform modifies the transitions originating from the start
 * state.  Every transition from the start state is modified to start
 * from the restore states added in either CMT2 or CMT3.
 *
 * Creation date: (12/28/00 12:07:00 AM)
 * @author: Athie Self
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */
 
import java.util.Vector;
import system.*;

public class CMTransform9 extends Transform {
	TSystem	currentSys = null;	// A pointer to the system being transformed
	TSystem beforeSys = null; 	// A copy of the system before the transformation
	TSystem sys = null;			// A copy of the system that is used to work on
/**
 * CMTransform9 constructor comment.
 */

public CMTransform9(TSystem t) {
	super();

	sys = t.copy();
	currentSys = t;
	beforeSys = t.copy();
	setRationale("");
	if(applicable(sys, null)){
		execute(null);
	}
}
/**
 * This method will only return true if the target is a TSystem 
 * and that system has roles and tasks.
 *
 * Creation date: (11/6/00 8:52:43 AM)
 */
public static boolean applicable(Object target, Object params) {
	TSystem t;
	if(target instanceof TSystem){
		t = (TSystem)target;
		if((t.roles != null)
			&& (t.tasks != null)){
				return true;
			}
		else {
			return false;
		}
	}
	else {
		return false;
	}
}
/**
 * the params attribute is not used
 */

public void execute(Object params) {	
			
	Vector agents = sys.agents;
	
	for(int i = 0; i < agents.size(); i++) {
	
		Agent ag = (Agent)agents.elementAt(i);

		if(isMobility_Specified(ag)) {
			
			Vector components = ag.getComponents();

			for(int j = 0; j < components.size(); j++) {

				Component c = (Component)components.elementAt(j);

				if(!(c.getName().equals("AgentComponent"))) {
					
					ATagent atAgent = ag.getAtAgent();
					ATarchitecture atArch = atAgent.getArchitecture();
					ATagentComponent atComponent = atArch.findComponentbyname(c.getName());

					StateTable st = c.getStateTable();
					ATstatetable atStateTable = atComponent.getStateTable();
					Vector transitions = st.getTransitions();

					// Go through all of the transitions that have a from state 
					// that is the start state and a to state that is different
					// from the restore state added and change the from state 
					// to the restore state and add a guard condition on the 
					// transition.
					for(int k = 0; k < transitions.size(); k++) {
				
						Transition t = (Transition)transitions.elementAt(k);
						
						if((t.getFromName().equals("StartState"))
							&& (!t.getToName().equals("restore"))) {

							ATtableElement atStartState = atStateTable.findStatebyname("StartState");
							ATtableElement atRestoreState = atStateTable.findStatebyname("restore");

							ATtransition atTrans = (ATtransition)t.getAtTransition();
							ATtableElement atNextState = atTrans.getNextState();
							atTrans.setCoordinates(atRestoreState,atNextState);
							atTrans.setGuardCondition("state=="+t.getToName());
								
							atComponent.setStateTable(atStateTable);
						}
					}					
				}
			}
		}
	}
		
	// now make the altered system the current system
	// can be undone by the unDo method
	currentSys = sys;
	
}
/**
 * Insert the method's description here.
 * Creation date: (12/29/2000 9:53:42 AM)
 * @return boolean
 * @param a Transforms.Agent
 */

public boolean isMobility_Specified(Agent a) {

	Vector actions    = new Vector();
	Vector states     = new Vector();
	
	Vector components = a.getComponents();
	
	if(components != null) {

		for(int j = 0; j < components.size(); j++) {

			Component c = (Component)components.elementAt(j);

			if (!c.getName().equals("AgentComponent")) {

				StateTable st = c.getStateTable();

				if(st != null) {
					states = st.getStates();
				}

				if(states != null) {					

					for(int k = 0; k < states.size(); k++) {

						State s = (State)states.elementAt(k);

						actions = s.getActions();

						if(actions != null) {

							for(int l = 0; l < actions.size(); l++) {

								Action act = (Action)actions.elementAt(l);

								FunctionCall fc = (FunctionCall)act.getFunction();

								if(fc!=null) {

									if(fc.getName().equals("move")) {
										return true;
									}
								}	
							}		
						}	
					}
				}
			}
		}
	}
	
	return false;	
}
/**
 * replay method comment.
 */
public void replay(Object top) {}
/**
 * unDo method comment.
 */
public void unDo(Object params) {
	currentSys = beforeSys;
}
}
