package Transforms;

/**
 * This class represents transformations 36 and 37 in the transformation process.
 * The result of this transform is that all receive events that have a parameter used
 * either in a transition or state that does not belong to the conversation will cause
 * an action to be added to the transition that sets it in the parent component.
 *
 * Creation date: (11/8/00 10:37:49 AM)
 * @author: Clint Sparkman
 *
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */
 
import java.util.Vector;
import java.util.StringTokenizer;
import java.io.StreamTokenizer;
import java.io.StringReader;

public class Transform36 extends Transform {
	TSystem	currentSys = null;	// A pointer to the system being transformed
	TSystem beforeSys = null; 	// A copy of the system before the transformation
	TSystem sys = null;			// A copy of the system that is used to work on
/**
 * Transform1 constructor comment.
 */
public Transform36(TSystem t) {
	super();

	sys = t;
	
	if(applicable(sys, null)){
		execute(null);
	}
}
/**
 * This method checks to see if the given parameter requires an action to
 * set it in the parent component.  The two conditions are that it is either used
 * in an action in a state not belonging to the conversations of t, or it is used
 * in a transition not belonging to the conversations of t.
 *
 * Creation date: (11/26/00 3:14:16 PM)
 */
boolean actionNeeded(StateTable st, Transition t, Parameter p) {
	Vector states = st.getStates();
	Vector transitions = st.getTransitions();
	Vector convs = t.getConversations();

	// check all of the states
	if(states != null)
	for(int i = 0; i < states.size(); i++){
		State s = (State)states.elementAt(i);
		Vector actions = s.getActions();
		if(actions != null){
			if(!s.hasAllConversations(convs)){
				// check each action in the state
				for(int j = 0; j < actions.size(); j++){
					Action a = (Action)actions.elementAt(j);
					if(usedInAction(p, a))
						return true;
				}
			}
		}
	}

	// check all of the transitions
	if(transitions != null)
	for(int i = 0; i < transitions.size(); i++){
		Transition t2 = (Transition)transitions.elementAt(i);
		if(!t2.hasAllConversations(convs) && usedInTransition(p, t2)){				
			return true;
		}
	}
		
	
	
	// default
	return false;
}
/**
 * This method should be overridden by the concrete subclass
 * to determine if the transform is applicable to the target.
 *
 * Creation date: (11/6/00 8:52:43 AM)
 */
public static boolean applicable(Object target, Object params) {
	TSystem t;
	if(target instanceof TSystem){
		t = (TSystem)target;
		if((t.roles != null)
			&& (t.tasks != null)
			&& (t.protocols != null)
			&& (t.agents != null)
			&& t.rolesCovered()
			&& t.noDuplicateTasks()){	
				return true;
			}
		else {
			return false;
		}
	}
	else {
		return false;
	}
}
/**
 * This method will look for receive events with parameters used outside
 * the conversation and will add an action to the transition to set it in the
 * parent component.
 *
 * the params attribute is not used
 */
public void execute(Object params) {	
	Vector 			agents = sys.agents;
	Agent 			a;
	Vector			components;
	Component		c;
	StateTable		st;
	Vector			transitions;
	Transition		t;

	if(agents != null)
	for(int i = 0; i < agents.size(); i++){
		a = (Agent)agents.elementAt(i);
		components = a.getComponents();
		if(components != null)
		for(int j = 0; j < components.size(); j++){
			c = (Component)components.elementAt(j);
			st = c.getStateTable();
			transitions = st.getTransitions();
			if(transitions != null)
			for(int k = 0; k < transitions.size(); k++){
				t = (Transition)transitions.elementAt(k);
				if((t.getConversations() != null) && (t.getReceiveEvent() != null)){
					ReceiveEvent re = t.getReceiveEvent();
					Event e = re.getEvent();
					Vector parameters = e.getParameters();
					if(parameters != null){
						for(int l = 0; l < parameters.size(); l++){
							Parameter p = (Parameter)parameters.elementAt(l);
							if(!isNumber(p.getName()) && actionNeeded(st, t, p)){
								// add the action to set the parameter
								Vector lhs = new Vector();
								Vector rhs = new Vector();
								lhs.add(new String("parent."+p.getName()));
								rhs.add(p.getName());
								Action act = new Action(lhs, rhs);
								if(t.getActions() == null)
									t.setActions(new Vector());
								t.getActions().add(act);
								t.addATActions();
							}						
						}
					}	
				}
			}
		}
	}	
}
/**
 * This method will check to see if the parameter is used somewhere in the guard
 * condition.  Since guard conditions are loosely defined as a string, the method
 * simply checks to see if it is a token in the string.
 *
 * Creation date: (11/26/00 3:14:16 PM)
 */
public static boolean inGuard(Parameter p, String guard) {
	String param = p.getName();
	StringTokenizer st = new StringTokenizer(guard, " ,=<>()");
	while(st.hasMoreTokens()){
		String token = st.nextToken();
		if(token.equals(param))
			return true;
	}
	
	// default
	return false;
}
/**
 * This method will check to see if the given string is a single number.
 *
 * Creation date: (11/26/00 3:14:16 PM)
 */
boolean isNumber(String s) {
	StringReader sr = new StringReader(s);
	StreamTokenizer st = new StreamTokenizer(sr);
	int count = 0;
	boolean number = false;
	
	try{
		st.nextToken();
	while(st.ttype != StreamTokenizer.TT_EOF){
		count++;
		if(st.ttype == StreamTokenizer.TT_NUMBER)
			number = true;
		st.nextToken();
	}
	}catch(Exception e){System.out.println(e);}

	if(number && (count == 1))
		return true;
	else		
		return false;
}
/**
 * replay method comment.
 */
public void replay(Object top) {}
/**
 * unDo method comment.
 */
public void unDo(Object params) {
	currentSys = beforeSys;
}
/**
 * This method checks to see if the parameter is used somewhere in the action.
 *
 * Creation date: (11/26/00 3:14:16 PM)
 */
boolean usedInAction(Parameter p, Action a) {
	Vector lhs = a.getLhs();
	Vector rhs = a.getRhs();
	FunctionCall f = a.getFunction();	
	
	if(lhs != null){
		for(int i = 0; i < lhs.size(); i++){
			String s = (String)lhs.elementAt(i);
			if(s.equals(p.getName()) 
				|| s.equals("parent." + p.getName()))
				return true;
		}
	}
	if(rhs != null){
		for(int i = 0; i < rhs.size(); i++){
			String s = (String)rhs.elementAt(i);
			if(s.equals(p.getName()) 
				|| s.equals("parent." + p.getName()))
				return true;
		}
	}
	if(f != null){
		Vector parameters = f.getParameters();
		if(parameters != null)
		for(int i = 0; i < parameters.size(); i++){
			Parameter p2 = (Parameter)parameters.elementAt(i);
			if(p2.getName().equals(p.getName()) 
				|| p2.getName().equals("parent." + p.getName()))
				return true;
		}
	}
		
	// default
	return false;
}
/**
 * This method checks to see if the parameter is used somewhere in the transition.
 *
 * Creation date: (11/26/00 3:14:16 PM)
 */
boolean usedInTransition(Parameter p, Transition t) {
	Parameter prepended = new Parameter("parent." + p.getName());
	
	Event receive = t.getReceive();
	if(receive != null){
		if(receive.hasParameter(p))
		return true;
	}
	if(t.getReceiveEvent() != null){
		ReceiveEvent re = t.getReceiveEvent();
		Event e = re.getEvent();
		if(e.hasParameter(p) || e.hasParameter(prepended))
		return true;
	}
	if((t.getGuard() != null) && inGuard(p, t.getGuard())){
		return true;		
	}
	Vector sends = t.getSends();
	if(sends != null){
		for(int x = 0; x < sends.size(); x++){
			Event e = (Event)sends.elementAt(x);
			if(e.hasParameter(p))
			return true;
		}
	}
	Vector sendEvents = t.getSendEvents();
	if(sendEvents != null){
		for(int x = 0; x < sendEvents.size(); x++){
			SendEvent se = (SendEvent)sendEvents.elementAt(x);
			Event e = se.getEvent();
			if(e.hasParameter(p) || e.hasParameter(prepended))
			return true;
		}
		
	}
	Vector actions = t.getActions();
	if(actions != null){
		for(int i = 0; i < actions.size(); i++){
			Action a = (Action)actions.elementAt(i);
			if(usedInAction(p, a) || usedInAction(prepended, a))
				return true;
		}
	}
		
	// default
	return false;
}
}
